
/**
 * @file   linux/at/at_asm_io.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <asm/io.h>
#include <linux/mm.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_asm_io.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

void *at_ioremap_nocache(uint64_t addr, unsigned long size_byte)
{
    at_assert(0 < size_byte);

    // ioremap has provided non-cached semantics by default since the Linux 2.6
    // days, so we should use ioremap_cache.
    return ioremap_cache(addr, size_byte);
}

// addr [---;---]
phys_addr_t at_virt_to_phys(volatile void *addr)
{
    at_assert(NULL != addr);
    return virt_to_phys(addr);
}
void *at_virt_to_page(volatile void *addr)
{
    at_assert(NULL != addr);
    return virt_to_page(addr);
}

// ----- IO port access -----------------------------------------------------

uint8_t at_inb(uint16_t port) { return inb(port); }
uint16_t at_inw(uint16_t port) { return inw(port); }
uint32_t at_inl(uint16_t port) { return inl(port); }

void at_outb(uint8_t value, uint16_t port) { outb(value, port); }
void at_outw(uint16_t value, uint16_t port) { outw(value, port); }
void at_outl(uint32_t value, uint16_t port) { outl(value, port); }

// ----- Memory access ------------------------------------------------------

uint16_t at_raw_readw(const volatile void *addr)
{
    at_assert(addr != NULL);
    return __raw_readw(addr);
}
uint32_t at_raw_readl(const volatile void *addr)
{
    at_assert(addr != NULL);
    return __raw_readl(addr);
}
uint64_t at_raw_readq(const volatile void *addr)
{
    at_assert(addr != NULL);
    return *(const volatile u64 __force *)addr;
}

uint8_t at_readb(const volatile void *addr)
{
    at_assert(addr != NULL);
    return readb(addr);
}
uint16_t at_readw(const volatile void *addr)
{
    at_assert(addr != NULL);
    return readw(addr);
}
uint32_t at_readl(const volatile void *addr)
{
    at_assert(addr != NULL);
    return readl(addr);
}
uint64_t at_readq(const volatile void *addr)
{
    at_assert(addr != NULL);
    return __le64_to_cpu(__raw_readq(addr));
}

void at_raw_writew(uint16_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    __raw_writew(value, addr);
}
void at_raw_writel(uint32_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    __raw_writel(value, addr);
}
void at_raw_writeq(uint64_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    *(volatile u64 __force *)addr = value;
}

void at_writeb(uint8_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    writeb(value, addr);
}
void at_writew(uint16_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    writew(value, addr);
}
void at_writel(uint32_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    writel(value, addr);
}
void at_writeq(uint64_t value, volatile void *addr)
{
    at_assert(addr != NULL);
    at_raw_writeq(__cpu_to_le64(value), addr);
}
