
/**
 * @file   linux/at/at_asmi_io.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// This file replace the kernel <asm/io.h> header.

#pragma once

// Functions
/////////////////////////////////////////////////////////////////////////////

// ===== The at public code defines =========================================

extern void *at_ioremap_nocache(uint64_t addr, unsigned long size_byte);

extern phys_addr_t at_virt_to_phys(volatile void *addr);
extern void *at_virt_to_page(volatile void *addr);

// ----- IO port access -----------------------------------------------------
extern uint8_t at_inb(uint16_t port);
extern uint32_t at_inl(uint16_t port);
extern uint16_t at_inw(uint16_t port);
extern void at_outb(uint8_t value, uint16_t port);
extern void at_outl(uint32_t value, uint16_t port);
extern void at_outw(uint16_t value, uint16_t port);

// ----- Memory access ------------------------------------------------------
extern uint16_t at_raw_readw(const volatile void *addr);
extern uint32_t at_raw_readl(const volatile void *addr);
extern uint64_t at_raw_readq(const volatile void *addr);
extern uint8_t at_readb(const volatile void *addr);
extern uint16_t at_readw(const volatile void *addr);
extern uint32_t at_readl(const volatile void *addr);
extern uint64_t at_readq(const volatile void *addr);
extern void at_raw_writew(uint16_t value, volatile void *addr);
extern void at_raw_writel(uint32_t value, volatile void *addr);
extern void at_raw_writeq(uint64_t value, volatile void *addr);
extern void at_writeb(uint8_t value, volatile void *addr);
extern void at_writew(uint16_t value, volatile void *addr);
extern void at_writel(uint32_t value, volatile void *addr);
extern void at_writeq(uint64_t value, volatile void *addr);
