
/**
 * @file   linux/at/at_asm_uaccess.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================

#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/version.h>

#include <asm/uaccess.h>

// ===== linux/at ===========================================================
#include <at_asm_uaccess.h>
#include <at_assert.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aAddr [---;---]
bool at_access_read_ok(void *aAddr, unsigned long aSize_byte)
{
    at_assert(NULL != aAddr);
#if defined(__range_not_ok)
    return likely(!__range_not_ok((long unsigned int)aAddr, aSize_byte, user_addr_max()));
#elif defined(__range_ok)
    return likely(__range_ok((long unsigned int)aAddr, aSize_byte));
#else
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
    return likely(access_ok(VERIFY_READ, (long unsigned int)aAddr, aSize_byte));
#else
    return likely(access_ok((long unsigned int)aAddr, aSize_byte));
#endif
#endif
}

// aAddr [---;---]
bool at_access_write_ok(void *aAddr, unsigned long aSize_byte)
{
    at_assert(NULL != aAddr);
#if defined(__range_not_ok)
    return likely(!__range_not_ok((long unsigned int)aAddr, aSize_byte, user_addr_max()));
#elif defined(__range_ok)
    return likely(__range_ok((long unsigned int)aAddr, aSize_byte));
#else
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
    return likely(access_ok(VERIFY_WRITE, (long unsigned int)aAddr, aSize_byte));
#else
    return likely(access_ok((long unsigned int)aAddr, aSize_byte));
#endif
#endif
}
