
/**
 * @file   linux/at/at_gate.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/spinlock.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_gate.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aThis [---;-W-]
void at_gate_init(struct at_gate *aThis)
{
    at_assert(NULL != aThis);

#ifdef CONFIG_PREEMPT_RT
    init_MUTEX(&aThis->mSemaphore);
#else
    spin_lock_init(&aThis->mSpinlock);
#endif
}

// aThis  [---;RW-]
// aLevel [---;-W-]
void at_gate_lock(struct at_gate *aThis, unsigned long *aLevel)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aLevel);

#ifdef CONFIG_PREEMPT_RT
    down(&aThis->mSemaphore);
    // QUESTION  Is the next line needed ?
    (*aLevel)++;
#else
    spin_lock_irqsave(&aThis->mSpinlock, *aLevel);
#endif
}

// aThis  [---;RW-]
// aLevel [---;R--]
void at_gate_unlock(struct at_gate *aThis, unsigned long *aLevel)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aLevel);

#ifdef CONFIG_PREEMPT_RT
    up(&aThis->mSemaphore);
    // QUESTION  Is the next line needed ?
    (*aLevel)--;
#else
    spin_unlock_irqrestore(&aThis->mSpinlock, *aLevel);
#endif
}

// aThis [---;-W-]
void at_gate_rt_init(struct at_gate_rt *aThis)
{
    at_assert(NULL != aThis);

#ifdef CONFIG_PREEMPT_RT
    init_MUTEX(&aThis->mSemaphore);
#endif
}

// aThis [---;-W-]
void at_gate_rt_down(struct at_gate_rt *aThis)
{
    at_assert(NULL != aThis);

#ifdef CONFIG_PREEMPT_RT
    down(&aThis->mSemaphore);
#endif
}

// aThis [---;-W-]
int at_gate_rt_down_trylock(struct at_gate_rt *aThis)
{
    at_assert(NULL != aThis);

#ifdef CONFIG_PREEMPT_RT
    return down_trylock(&aThis->mSemaphore);
#else
    return 0;
#endif
}

// aThis [---;-W-]
void at_gate_rt_up(struct at_gate_rt *aThis)
{
    at_assert(NULL != aThis);

#ifdef CONFIG_PREEMPT_RT
    up(&aThis->mSemaphore);
#endif
}
