
/**
 * @file   linux/at/at_linux_dma-mapping.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/dma-mapping.h>

// ===== linux/at ===========================================================
#include <at_assert.h>
#include <at_linux_dma-mapping.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aDevice [---;RW-]
// aOut    [---;-W-]
void *at_dma_alloc_coherent(struct device *aDevice, size_t aSize_byte, dma_addr_t *aOut)
{
    at_assert(NULL != aDevice);
    at_assert(0 < aSize_byte);
    at_assert(NULL != aOut);

    return dma_alloc_coherent(aDevice, aSize_byte, aOut, GFP_KERNEL | __GFP_NOWARN);
}

// aDevice [---;RW-]
// aBuffer [D--;---]
void at_dma_free_coherent(struct device *aDevice,
                          size_t aSize_byte,
                          void *aBuffer,
                          dma_addr_t aBuffer_PA)
{
    at_assert(NULL != aDevice);
    at_assert(0 < aSize_byte);
    at_assert(NULL != aBuffer);

    dma_free_coherent(aDevice, aSize_byte, aBuffer, aBuffer_PA);
}

// aDevice [---;RW-]
// aPage   [---;RW-]
dma_addr_t at_dma_map_page(struct device *aDevice,
                           struct page *aPage,
                           size_t aOffset,
                           size_t aSize,
                           enum dma_data_direction aDir)
{
    at_assert(NULL != aDevice);
    at_assert(NULL != aPage);
    at_assert(0 < aSize);

    return dma_map_page(aDevice, aPage, aOffset, aSize, aDir);
}

// aDevice [---;RW-]
int at_dma_set_mask(struct device *aDevice, uint64_t aMask)
{
    at_assert(NULL != aDevice);

    return dma_set_mask(aDevice, aMask);
}

// aDevice [---;RW-]
void at_dma_unmap_page(struct device *aDevice,
                       size_t aOffset,
                       size_t aSize,
                       enum dma_data_direction aDir)
{
    at_assert(NULL != aDevice);
    at_assert(0 < aSize);

    dma_unmap_page(aDevice, aOffset, aSize, aDir);
}
