
/**
 * @file   linux/at/at_linux_dma-mapping.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// This file replace the kernel <linux/dma-mapping.h> header.

#pragma once

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
struct device;
struct page;

// Data type
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

enum dma_data_direction
{
    DMA_BIDIRECTIONAL = 0,
    DMA_TO_DEVICE = 1,
    DMA_FROM_DEVICE = 2,
    DMA_NONE = 3,
};

#endif

// Functions
/////////////////////////////////////////////////////////////////////////////

// ===== The at public code defines =========================================
extern void *at_dma_alloc_coherent(struct device *aDevice, size_t aSize_byte, dma_addr_t *aOut);
extern void at_dma_free_coherent(struct device *aDevice,
                                 size_t aSize_byte,
                                 void *aBuffer,
                                 dma_addr_t aBuffer_PA);
extern dma_addr_t at_dma_map_page(struct device *aDevice,
                                  struct page *aPage,
                                  size_t aOffset,
                                  size_t aSize,
                                  enum dma_data_direction aDir);
extern int at_dma_set_mask(struct device *aDevice, uint64_t aMask);
extern void at_dma_unmap_page(struct device *aDevice,
                              size_t aOffset,
                              size_t aSize,
                              enum dma_data_direction aDir);
