
/**
 * @file   linux/at/at_linux_fs.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/fs.h>
#include <linux/version.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_fs.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aThis [---;-W-]
void at_file_clear_private_data(struct file *aThis)
{
    at_assert(NULL != aThis);

    aThis->private_data = NULL;
}

// aThis [---;R--]
struct inode *at_file_get_inode(struct file *aThis)
{
    at_assert(NULL != aThis);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 19, 0)
    return file_inode(aThis);
#else
    return aThis->f_dentry->d_inode;
#endif
}

// aThis [---;R--]
void *at_file_get_private_data(struct file *aThis)
{
    at_assert(NULL != aThis);

    return aThis->private_data;
}

// aThis [---;-W-]
// aData [-KO;---]
void at_file_set_private_data(struct file *aThis, void *aData)
{
    at_assert(NULL != aThis);

    aThis->private_data = aData;
}
