
/**
 * @file   linux/at/at_interrupt.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// ===== Linux kernel =======================================================
#include <linux/interrupt.h>

// ===== linux/at ===========================================================
#include <at_assert.h>
#include <at_gate.h>

#include <at_linux_interrupt.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

#undef AT_DEBUG
#ifdef AT_DEBUG
#define DBG_PRINT(...) printk(KERN_DEBUG __VA_ARGS__)
#else
#define DBG_PRINT(...)
#endif

// aDev [-KO;---]
void at_irq_free(unsigned int aIrq, void *aDev)
{
    DBG_PRINT("%s(%u, %p)\n", __FUNCTION__, aIrq, aDev);
    free_irq(aIrq, aDev);
}

// aHandler [---;--X]
// aName    [-K-;R--]
// aDev     [-KO;---]
int at_irq_request(
    unsigned int aIrq, irq_handler_t aHandler, bool aShared, const char *aName, void *aDev)
{
    DBG_PRINT("%s(%u, %p, %u, %s, %p)\n", __FUNCTION__, aIrq, aHandler, aShared, aName, aDev);
    at_assert(NULL != aHandler);
    at_assert(NULL != aName);
    return request_irq(aIrq, aHandler, aShared ? IRQF_SHARED : 0, aName, aDev);
}

void at_irq_enable(unsigned int aIrq)
{
    enable_irq(aIrq);
}

void at_irq_disable(unsigned int aIrq)
{
    disable_irq(aIrq);
}

// aThis [---;RW-]
void at_tasklet_kill(struct tasklet_struct *aThis)
{
    DBG_PRINT("%s(%p)\n", __FUNCTION__, aThis);
    at_assert(NULL != aThis);
    tasklet_kill(aThis);
}

// aThis [---;-W-]
// aFunc [-K-;--X]
void at_tasklet_init(struct tasklet_struct *aThis,
                     void (*aFunc)(unsigned long),
                     unsigned long aData)
{
    DBG_PRINT("%s(%p, %p)\n", __FUNCTION__, aThis, aFunc);
    at_assert(NULL != aThis);
    at_assert(NULL != aFunc);
    tasklet_init(aThis, aFunc, aData);
}

// aThis [---;RW-]
void at_tasklet_schedule(struct tasklet_struct *aThis)
{
    DBG_PRINT("%s(%p)\n", __FUNCTION__, aThis);
    at_assert(NULL != aThis);
    tasklet_schedule(aThis);
}

// aThis [---;RW-]
// aGate [---;RW-]
void at_tasklet_schedule_gated(struct tasklet_struct *aThis, struct at_gate_rt *aGate)
{
    DBG_PRINT("%s(%p, %p)\n", __FUNCTION__, aThis, aGate);
    at_assert(NULL != aThis);
    at_assert(NULL != aGate);
    at_gate_rt_down(aGate);
    at_tasklet_schedule(aThis);
    at_gate_rt_up(aGate);
}
