
/**
 * @file   linux/at/at_interrupt.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// This file replace the kernel <linux/interrupt.h> header.

#pragma once

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== linux/at ===========================================================
struct at_gate_rt;

// Data types
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

enum irqreturn
{
    IRQ_NONE = (0 << 0),
    IRQ_HANDLED = (1 << 0),
    IRQ_WAKE_THREAD = (1 << 1),
};

typedef enum irqreturn irqreturn_t;

typedef irqreturn_t (*irq_handler_t)(int, void *);

struct tasklet_struct
{
    // Must be aligned to 64 bits
    uint64_t mReserved0[8];
};

#endif

// Functions
/////////////////////////////////////////////////////////////////////////////

// ===== The at public code defines =========================================

extern void at_irq_free(unsigned int aIrq, void *aDev);
extern int at_irq_request(
    unsigned int aIrq, irq_handler_t aHandler, bool aShared, const char *aName, void *aDev);
extern void at_irq_enable(unsigned int aIrq);
extern void at_irq_disable(unsigned int aIrq);
extern void
at_tasklet_init(struct tasklet_struct *aThis, void (*aFunc)(unsigned long), unsigned long aData);
extern void at_tasklet_kill(struct tasklet_struct *aThis);
extern void at_tasklet_schedule(struct tasklet_struct *aThis);
extern void at_tasklet_schedule_gated(struct tasklet_struct *aThis, struct at_gate_rt *aGate);
