
/**
 * @file   linux/at/at_linux_list.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/list.h>
#include <linux/spinlock.h>

// ===== linux/at ===========================================================
#include <at_assert.h>
#include <at_gate.h>

#include <at_linux_list.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aAdd  [---;-W-]
// aThis [---;RW-]
void at_list_add_tail(struct list_head *aAdd, struct list_head *aThis)
{
    at_assert(NULL != aAdd);
    at_assert(NULL != aThis);

    list_add_tail(aAdd, aThis);
}

// aAdd  [---;-W-]
// aThis [---;RW-]
// aGate [---;RW-]
void at_list_add_tail_gated(struct list_head *aAdd, struct list_head *aThis, struct at_gate *aGate)
{
    unsigned long lLock;

    at_assert(NULL != aAdd);
    at_assert(NULL != aThis);
    at_assert(NULL != aGate);

    at_gate_lock(aGate, &lLock);
    at_list_add_tail(aAdd, aThis);
    at_gate_unlock(aGate, &lLock);
}

// aDel [---;RW-]
void at_list_del(struct list_head *aDel)
{
    at_assert(NULL != aDel);

    list_del(aDel);
}

// aThis [---;R--]
int at_list_empty(struct list_head *aThis)
{
    at_assert(NULL != aThis);
    return list_empty(aThis);
}
int at_list_empty_careful(struct list_head *aThis)
{
    at_assert(NULL != aThis);
    return list_empty_careful(aThis);
}

// aThis [---;R--]
// aGate [---;RW-]
int at_list_empty_gated(struct list_head *aThis, struct at_gate *aGate)
{
    unsigned long lLock;
    int lResult;

    at_assert(NULL != aThis);
    at_assert(NULL != aGate);

    at_gate_lock(aGate, &lLock);
    lResult = list_empty(aThis);
    at_gate_unlock(aGate, &lLock);

    return lResult;
}

// aThis [---;-W-]
void at_list_init(struct list_head *aThis)
{
    at_assert(NULL != aThis);

    INIT_LIST_HEAD(aThis);
}

// aMove [---;RW-]
// aThis [---;RW-]
void at_list_move_tail(struct list_head *aMove, struct list_head *aThis)
{
    at_assert(NULL != aMove);
    at_assert(NULL != aThis);

    list_move_tail(aMove, aThis);
}
