
/**
 * @file   linux/at/at_linux_list.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// This file replace the kernel <linux/list.h> header.

#pragma once

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== linux/at ===========================================================
struct at_gate;

// Data types
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

struct list_head
{
    struct list_head *next, *prev;
};

#endif

// Macros
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

#define container_of(P, T, M)                                                                      \
    ({                                                                                             \
        const typeof(((T *)0)->M) *__mptr = (P);                                                   \
        (T *)((char *)__mptr - offsetof(T, M));                                                    \
    })

#endif

#define at_list_entry(P, T, M) container_of(P, T, M)

#define at_list_first_entry(P, T, M) at_list_entry((P)->next, T, M)

#define at_list_next_entry(P, M) at_list_entry((P)->M.next, typeof(*(P)), M)

#define at_list_for_each(P, H) for ((P) = (H)->next; (P) != (H); (P) = (P)->next)

#define at_list_for_each_entry(P, H, M)                                                            \
    for (P = at_list_first_entry(H, typeof(*P), M); &P->M != (H); P = at_list_next_entry(P, M))

// Functions
/////////////////////////////////////////////////////////////////////////////

// ===== The at public code defines =========================================
extern void at_list_add_tail(struct list_head *aAdd, struct list_head *aThis);
extern void
at_list_add_tail_gated(struct list_head *aAdd, struct list_head *aThis, struct at_gate *aGate);
extern void at_list_del(struct list_head *aDel);
extern int at_list_empty(struct list_head *aThis);
extern int at_list_empty_gated(struct list_head *aThis, struct at_gate *aGate);
extern int at_list_empty_careful(struct list_head *aThis);
extern void at_list_init(struct list_head *aThis);
extern void at_list_move_tail(struct list_head *aMove, struct list_head *aThis);
