
/**
 * @file   linux/at/at_linux_mempool.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/mempool.h>
#include <linux/slab.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_mempool.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aName [---;R--]
struct kmem_cache *at_kmem_cache_create(const char *aName, unsigned int aSize)
{
    at_assert(NULL != aName);
    at_assert(0 < aSize);

    return kmem_cache_create(aName, aSize, 0, SLAB_HWCACHE_ALIGN, NULL);
}

// aThis [D--;RW-]
void at_kmem_cache_destroy(struct kmem_cache *aThis)
{
    at_assert(NULL != aThis);

    kmem_cache_destroy(aThis);
}

// aCache [-K-;RW-]
mempool_t *at_mem_pool_create_slab(int aMin, struct kmem_cache *aCache)
{
    at_assert(0 < aMin);
    at_assert(NULL != aCache);

    return mempool_create_slab_pool(aMin, aCache);
}

// aThis [---;RW-]
void *at_mem_pool_alloc(mempool_t *aThis)
{
    at_assert(NULL != aThis);

    return mempool_alloc(aThis, GFP_KERNEL);
}

// aThis [D--;RW-]
void at_mem_pool_destroy(mempool_t *aThis)
{
    at_assert(NULL != aThis);

    mempool_destroy(aThis);
}

// aElement [D--;---]
// aThis    [---;RW-]
void at_mem_pool_free(void *aElement, mempool_t *aThis)
{
    at_assert(NULL != aElement);
    at_assert(NULL != aThis);

    mempool_free(aElement, aThis);
}
