
/**
 * @file   linux/at/at_linux_mm.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// ===== Linux kernel =======================================================
#include <linux/mm.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_mm.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

uint32_t at_page_align(uint32_t aSize_byte) { return PAGE_ALIGN(aSize_byte); }

// aAddress [---;---]
uint64_t at_page_get_first(void *aAddress)
{
    at_assert(NULL != aAddress);

    return ((unsigned long)aAddress & PAGE_MASK) >> PAGE_SHIFT;
}

// aAddress [---;---]
uint64_t at_page_get_last(void *aAddress, size_t aSize_byte)
{
    at_assert(NULL != aAddress);
    at_assert(0 < aSize_byte);

    return (((unsigned long)aAddress + aSize_byte - 1) & PAGE_MASK) >> PAGE_SHIFT;
}

uint64_t at_page_get_mask() { return PAGE_MASK; }

// aAddress [---;---]
uint32_t at_page_get_offset(void *aAddress)
{
    at_assert(NULL != aAddress);

    return offset_in_page(aAddress);
}

uint32_t at_page_get_size() { return PAGE_SIZE; }

uint64_t at_page_get_shift() { return PAGE_SHIFT; }

// aAddress [---;---]
// aPages   [---;-W-]
int at_page_get_user_pages_fast(void *aAddress,
                                uint32_t aPageCount,
                                bool aReadFromCard,
                                struct page **aPages)
{
    // printk(KERN_DEBUG "at_page_get_user_pages_fast( , %u, %s,  )\n",
    // aPageCount, aReadFromCard ? "true" : "false");

    at_assert(NULL != aAddress);
    at_assert(0 < aPageCount);
    at_assert(NULL != aPages);

    return get_user_pages_fast((unsigned long)(aAddress), aPageCount, aReadFromCard, aPages);
}

// aThis [---;RW-]
void at_page_clear_reserved(struct page *aThis)
{
    at_assert(NULL != aThis);

    ClearPageReserved(aThis);
}

// aThis [---;R--]
int at_page_is_reserved(struct page *aThis)
{
    at_assert(NULL != aThis);

    return PageReserved(aThis);
}

// aThis [---;RW-]
void at_page_put(struct page *aThis)
{
    at_assert(NULL != aThis);

    put_page(aThis);
}

// aThis [---;RW-]
void at_page_set_dirty(struct page *aThis)
{
    at_assert(NULL != aThis);

    SetPageDirty(aThis);
}

// aThis [---;RW-]
void at_page_set_reserved(struct page *aThis)
{
    at_assert(NULL != aThis);

    SetPageReserved(aThis);
}

// aList [---;R--] [---;RW-]
extern void at_page_list_put(struct page **aList, unsigned int aCount)
{
    unsigned int i;

    at_assert(NULL != aList);
    at_assert(0 < aCount);

    for (i = 0; i < aCount; i++)
    {
        at_assert(NULL != aList[i]);

        put_page(aList[i]);
    }
}

// aList [--O;R--] [---;RW-]
extern void at_page_list_set_dirty(struct page **aList, unsigned int aCount)
{
    unsigned int i;

    // printk(KERN_DEBUG "at_page_list_set_dirty( , %u )\n", aCount);

    for (i = 0; i < aCount; i++)
    {
        at_assert(aList[i] != NULL);

        if (!PageReserved(aList[i])) { SetPageDirty(aList[i]); }
    }
}
