
/**
 * @file   linux/at/at_linux_pci.h
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// This file replace the kernel <linux/pci.h> header.

#pragma once

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
struct msix_entry;
struct pci_dev;

// Constants
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

#define DEVICE_COUNT_RESOURCE (4)

#define MAX_BARS (6)

#define PCI_VENDOR_ID (0x00)
#define PCI_DEVICE_ID (0x02)
#define PCI_COMMAND (0x04)
#define PCI_STATUS (0x06)
#define PCI_CACHE_LINE_SIZE (0x0c)
#define PCI_LATENCY_TIMER (0x0d)
#define PCI_HEADER_TYPE (0x0e)
#define PCI_BASE_ADDRESS_0 (0x10)
#define PCI_BASE_ADDRESS_1 (0x14)
#define PCI_BASE_ADDRESS_2 (0x18)
#define PCI_BASE_ADDRESS_3 (0x1c)
#define PCI_BASE_ADDRESS_4 (0x20)
#define PCI_BASE_ADDRESS_5 (0x24)
#define PCI_CARDBUS_CIS (0x28)
#define PCI_SUBSYSTEM_VENDOR_ID (0x2c)
#define PCI_SUBSYSTEM_ID (0x2e)
#define PCI_ROM_ADDRESS (0x30)
#define PCI_CAPABILITY_LIST (0x34)
#define PCI_INTERRUPT_LINE (0x3c)
#define PCI_INTERRUPT_PIN (0x3d)
#define PCI_MIN_GNT (0x3e)
#define PCI_MAX_LAT (0x3f)

#define PCI_STANDARD_SIZE (64)

#define PCI_CAP_ID_MSI (0x05)
#define PCI_CAP_ID_MSIX (0x11)

#define PCI_MSI_FLAGS_ENABLE (0x0001)
#define PCI_MSI_FLAGS_QMASK (0x000e)
#define PCI_MSI_FLAGS_QSIZE (0x0070)
#define PCI_MSI_FLAGS_64BIT (0x0080)

#define PCI_DEVICE_SPECIFIC_SIZE (192)

#endif

// Data types
/////////////////////////////////////////////////////////////////////////////

#ifndef MODULE

struct msix_entry
{
    // Must be aligned to 64 bits
    uint64_t mReserved0[1];
};

typedef struct
{
    uint16_t VendorId;
    uint16_t DeviceId;
    uint16_t Command;
    uint16_t Status;
    uint8_t RevisionId;
    uint8_t Interface;
    uint8_t SubClass;
    uint8_t BaseClass;
    uint8_t CacheLineSize;
    uint8_t LatencyTimer;
    uint8_t HeaderType;
    uint8_t BIST;
    uint32_t BarCfg[MAX_BARS];
    uint32_t CardBusCISPtr;
    uint16_t SubsystemVendorId;
    uint16_t SubsystemId;
    uint32_t ExpRomCfg;
    uint8_t CapabilitiesPtr;
    uint8_t Reserved1[3];
    uint32_t Reserved2[1];
    uint8_t InterruptLine;
    uint8_t InterruptPin;
    uint8_t MinimumGrant;
    uint8_t MaximumLatency;
    uint8_t DeviceSpecific[PCI_DEVICE_SPECIFIC_SIZE];
} __attribute__((packed)) PCI_CONFIG_HEADER;

#endif

// Functions
/////////////////////////////////////////////////////////////////////////////

// ===== The public code defines ============================================

extern void *
at_pci_alloc_consistent(struct pci_dev *aPciDev, size_t aSize_byte, dma_addr_t *aDmaAddr);
extern bool at_pci_alloc_irq_vectors_MSI(struct pci_dev *aPciDev, unsigned int aCount);
extern bool at_pci_alloc_irq_vectors_MSIX(struct pci_dev *aPciDev,
                                          unsigned int aCount,
                                          struct msix_entry *aEntries);
extern void at_pci_disable_device(struct pci_dev *aPciDev);
extern void at_pci_disable_msi(struct pci_dev *aPciDev);
extern void at_pci_disable_msix(struct pci_dev *aPciDev);
extern int at_pci_enable_device(struct pci_dev *aPciDev);
extern int at_pci_enable_msi(struct pci_dev *aPciDev);
extern int at_pci_find_capability(struct pci_dev *aPciDev, int aCap);
extern void at_pci_free_consistent(struct pci_dev *aPciDev,
                                   uint64_t aSize_byte,
                                   void *aVirtAddr,
                                   uint64_t aHandle);
extern void at_pci_free_irq_vectors_MSI(struct pci_dev *aPciDev);
extern uint8_t at_pci_get_bus_number(struct pci_dev *aPciDev);
extern uint32_t at_pci_get_class(struct pci_dev *aPciDev);
extern struct device *at_pci_get_device(struct pci_dev *aPciDev);
extern uint16_t at_pci_get_device_id(struct pci_dev *aPciDev);
extern unsigned int at_pci_get_irq(struct pci_dev *aPciDev);
extern uint8_t at_pci_get_revision(struct pci_dev *aPciDev);
extern uint8_t at_pci_get_slot(struct pci_dev *aPciDev);
extern uint16_t at_pci_get_vendor_id(struct pci_dev *aPciDev);
extern int at_pci_irq_vector(struct pci_dev *aPciDev, unsigned int aIndex);
extern bool at_pci_is_irq_zero(struct pci_dev *aPciDev);
extern void *at_pci_ioremap(struct pci_dev *aPciDev, unsigned int aIndex);
extern void at_pci_iounmap(void *aVirtAddr);
extern uint32_t
at_pci_map_sg(struct pci_dev *aPciDev, void *aSgList, uint32_t aSgCount, bool aReadFromCard);
extern int at_pci_read_config_byte(struct pci_dev *aPciDev, int aWhere, uint8_t *aValue);
extern int at_pci_read_config_bytes(struct pci_dev *aPciDev,
                                    int aWhere,
                                    uint8_t *aValues,
                                    unsigned int aCount);
extern int at_pci_read_config_dword(struct pci_dev *aPciDev, int aWhere, uint32_t *aValue);
extern int at_pci_read_config_word(struct pci_dev *aPciDev, int aWhere, uint16_t *aValue);
extern void at_pci_release_regions(struct pci_dev *aPciDev);
extern int at_pci_request_regions(struct pci_dev *aPciDev);
extern bool at_pci_resource_io(struct pci_dev *aPciDev, unsigned int aIndex);
extern uint64_t at_pci_resource_len(struct pci_dev *aPciDev, unsigned int aIndex);
extern bool at_pci_resource_mem(struct pci_dev *aPciDev, unsigned int aIndex);
extern uint64_t at_pci_resource_start(struct pci_dev *aPciDev, unsigned int aIndex);
extern int at_pci_set_dma_mask(struct pci_dev *aPciDev, unsigned int aBits);
extern void at_pci_set_master(struct pci_dev *aPciDev);
extern void
at_pci_unmap_sg(struct pci_dev *aPciDev, void *aSgList, uint32_t aSgCount, bool aReadFromCard);
extern int at_pci_write_config_byte(struct pci_dev *aPciDev, int aWhere, uint8_t aValue);
extern int at_pci_write_config_dword(struct pci_dev *aPciDev, int aWhere, uint32_t aValue);

extern unsigned int at_pci_msix_entry_get_entry(struct msix_entry *aEntries, unsigned int aIndex);
extern unsigned int at_pci_msix_entry_get_vector(struct msix_entry *aEntries, unsigned int aIndex);
extern void
at_pci_msix_entry_set_entry(struct msix_entry *aEntries, unsigned int aIndex, unsigned int aEntry);

extern struct pci_dev *
at_pci_retrieve_device(unsigned int aVendor, unsigned int aDevice, struct pci_dev *aFrom);
