
/**
 * @file   linux/at/at_linux_proc_fs.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/proc_fs.h>
#include <linux/version.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_proc_fs.h>

// Static function declarations
/////////////////////////////////////////////////////////////////////////////

#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 0)
static void RemoveSubdir(struct proc_dir_entry *aFirst);
#endif

// Functions
/////////////////////////////////////////////////////////////////////////////

// aThis [---;R--]
void *at_inode_get_data(struct inode *aThis)
{
    at_assert(NULL != aThis);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0)
    return pde_data(aThis);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)
    return PDE_DATA(aThis);
#else
    return PDE(aThis)->data;
#endif
}

// aThis [---;R--]
uint32_t at_inode_get_minor(struct inode *aThis)
{
    at_assert(NULL != aThis);

    return iminor(aThis);
}

// aName   [---;R--]
// aParent [--O;RW-]
struct proc_dir_entry *at_proc_mkdir(const char *aName, struct proc_dir_entry *aParent)
{
    // printk(KERN_DEBUG "at_proc_mkdir( \"%s\",  )\n", aName);

    at_assert(NULL != aName);

    return proc_mkdir(aName, aParent);
}

// aThis [D--;RW-]
void at_proc_remove(struct proc_dir_entry *aThis)
{
    // printk(KERN_DEBUG "at_proc_remove(  )\n");

    at_assert(NULL != aThis);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)

    proc_remove(aThis);

#else

    if (NULL != aThis->subdir) { RemoveSubdir(aThis->subdir); }

    if (NULL != aThis->data)
    {
        ats_free(aThis->data);
        aThis->data = NULL;
    }

    remove_proc_entry(aThis->name, aThis->parent);

#endif
}

// Static functions
/////////////////////////////////////////////////////////////////////////////

#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 0)

// aFirst [D--;RW-]
void RemoveSubdir(struct proc_dir_entry *aFirst)
{
    // printk(KERN_DEBUG "RemoveSubdir(  )\n");

    at_assert(NULL != aFirst);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 10, 0)

    proc_remove(aFirst);

#else

    struct proc_dir_entry *lCurrent = aFirst;

    do
    {
        struct proc_dir_entry *lNext = lCurrent->next;

        at_proc_remove(lCurrent);

        lCurrent = lNext;
    } while (NULL != lCurrent);

#endif
}

#endif
