
/**
 * @file   linux/at/at_linux_scatterlist.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/scatterlist.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_scatterlist.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aThis [---;-W-]
// aPage [---;R--]
void at_scatter_list_assign_page(struct scatterlist *aThis, struct page *aPage)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aPage);

    sg_assign_page(aThis, aPage);
}

// aThis [---;R--]
uint64_t at_scatter_list_get_dma_address(struct scatterlist *aThis)
{
    uint64_t lResult;

    at_assert(NULL != aThis);

    lResult = sg_dma_address(aThis);

    // printk(KERN_DEBUG "at_scatter_list_get_dma_address(  ) - 0x%016lx\n",
    // lResult);

    return lResult;
}

// aThis [---;R--]
uint32_t at_scatter_list_get_dma_len(struct scatterlist *aThis)
{
    uint32_t lResult;

    at_assert(NULL != aThis);

    lResult = sg_dma_len(aThis);

    // printk(KERN_DEBUG "at_scatter_list_get_dma_len(  ) - %u bytes\n",
    // lResult);

    return lResult;
}

// aThis [---;---]
void *at_scatter_list_get_element(struct scatterlist *aList, uint32_t aIndex)
{
    struct scatterlist *lResult = aList;

    // printk(KERN_DEBUG "at_scatter_list_get_element( , %u )\n", aIndex);

    at_assert(NULL != aList);

    lResult += aIndex;

    return lResult;
}

uint32_t at_scatter_list_get_element_size() { return sizeof(struct scatterlist); }

// aThis [---;R--]
uint32_t at_scatter_list_get_length(struct scatterlist *aThis)
{
    at_assert(NULL != aThis);

    return aThis->length;
}

// aThis [---;R--]
uint32_t at_scatter_list_get_offset(struct scatterlist *aThis)
{
    at_assert(NULL != aThis);

    return aThis->offset;
}

// aList [---;-W-]
void at_scatter_list_init(struct scatterlist *aList, uint32_t aCount)
{
    at_assert(NULL != aList);
    at_assert(0 < aCount);

    sg_init_table(aList, aCount);
}

// aThis [---;-W-]
void at_scatter_list_set_length(struct scatterlist *aThis, uint32_t aLength)
{
    at_assert(NULL != aThis);

    aThis->length = aLength;
}

// aThis [---;-W-]
void at_scatter_list_set_offset(struct scatterlist *aThis, uint32_t aOffset)
{
    at_assert(NULL != aThis);

    aThis->offset = aOffset;
}

// aThis [---;RW-]
// aPage [-K-;---]
void at_scatter_list_set_page(struct scatterlist *aThis,
                              struct page *aPage,
                              unsigned int aLength,
                              unsigned int aOffset)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aPage);

    sg_set_page(aThis, aPage, aLength, aOffset);
}
