
/**
 * @file   linux/at/at_linux_wait.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/sched.h>
#include <linux/wait.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_linux_wait.h>

// Functions
/////////////////////////////////////////////////////////////////////////////

// aThis [---;-W-]
void at_wait_queue_init(wait_queue_head_t *aThis)
{
    at_assert(NULL != aThis);

    init_waitqueue_head(aThis);
}

// aThis [---;RW-]
// aFlag [---;R--]
int at_wait_queue_wait_event_interruptible(wait_queue_head_t *aThis, uint16_t *aFlag)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aFlag);

    return wait_event_interruptible(*aThis, *aFlag);
}

// aThis [---;RW-]
// aFlag [---;R--]
int at_wait_queue_wait_event_interruptible_timeout(wait_queue_head_t *aThis,
                                                   uint16_t *aFlag,
                                                   int64_t aTimeout)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aFlag);

    return wait_event_interruptible_timeout(*aThis, *aFlag, aTimeout);
}

// aThis [---;RW-]
// aFlag [---;R--]
int at_wait_queue_wait_event_interruptible_timeout_eq(wait_queue_head_t *aThis,
                                                      int *aFlag,
                                                      int aValue,
                                                      int64_t aTimeout)
{
    at_assert(NULL != aThis);
    at_assert(NULL != aFlag);

    return wait_event_interruptible_timeout(*aThis, *aFlag == aValue, aTimeout);
}

// aThis [---;RW-]
void at_wait_queue_wake_up(wait_queue_head_t *aThis)
{
    at_assert(NULL != aThis);

    wake_up(aThis);
}

// aThis [---;RW-]
void at_wait_queue_wake_up_interruptible(wait_queue_head_t *aThis)
{
    at_assert(NULL != aThis);

    wake_up_interruptible(aThis);
}
