
/**
 * @file   linux/at/at_main_B.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/version.h>

// ===== linux/at ===========================================================
#include <at_assert.h>
#include <at_linux_proc_fs.h>
#include <at_main.h>

#include <at_main_B.h>

// ===== src/ats... =========================================================
#include <DeviceId.h>

// Macros
/////////////////////////////////////////////////////////////////////////////

#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 8, 0)
#define DEV_EXIT __devexit
#define DEV_INIT __devinit
#else
#define DEV_EXIT
#define DEV_INIT
#endif

// Static function declarations
/////////////////////////////////////////////////////////////////////////////

static void __exit ModuleExit(void);
static int __init ModuleInit(void);

// ===== Old kernel support =================================================

#if defined(CONFIG_COMPAT_ENABLED) && defined(CONFIG_COMPAT)

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
static int ConvertCompatIoCtl(struct inode *aINnode,
                              struct file *aFile,
                              unsigned int aCmd,
                              unsigned long aArgs);
#endif

#endif

// Module definition
/////////////////////////////////////////////////////////////////////////////

module_exit(ModuleExit);
module_init(ModuleInit);

// Static variables
/////////////////////////////////////////////////////////////////////////////

static struct file_operations sFileOperations;

static int sMajor;

// Static functions
/////////////////////////////////////////////////////////////////////////////

void __exit ModuleExit()
{
    // printk(KERN_DEBUG "ModuleExit()\n");

    at_module_exit();

    unregister_chrdev(sMajor, DRIVER_NAME);

    sMajor = 0;
}

int __init ModuleInit()
{
    int lResult;

    // printk(KERN_DEBUG "ModuleInit()\n");

    sFileOperations.owner = THIS_MODULE;

    sFileOperations.open = at_dispatch_open;
    sFileOperations.release = at_dispatch_release;
    sFileOperations.unlocked_ioctl = at_dispatch_ioctl;

#if defined(CONFIG_COMPAT_ENABLED) && defined(CONFIG_COMPAT)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)
    sFileOperations.compat_ioctl = ConvertCompatIoCtl;
#else
    sFileOperations.compat_ioctl = at_dispatch_compat_ioctl;
#endif
#endif

    sMajor = register_chrdev(0, DRIVER_NAME, &sFileOperations);

    lResult = at_module_init(sMajor);
    if (0 != lResult) { unregister_chrdev(sMajor, DRIVER_NAME); }

    return lResult;
}

// ===== Old kernel support =================================================

#if defined(CONFIG_COMPAT_ENABLED) && defined(CONFIG_COMPAT)

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 35)

// aINode [---;---]
// aFile  [---;---]
int ConvertCompatIoCtl(struct inode *aINnode,
                       struct file *aFile,
                       unsigned int aCmd,
                       unsigned long aArgs)
{
    at_assert(NULL != aINode);
    at_assert(NULL != aFile);

    at_dispatch_compat_ioctl(aFile, aCmd, aArgs);
}

#endif

#endif
