
/**
 * @file   linux/at/at_params.c
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2020 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/module.h>
#include <linux/version.h>

// ===== linux/at ===========================================================
#include <at_assert.h>

#include <at_params.h>

// Constants
/////////////////////////////////////////////////////////////////////////////

#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 15, 0)
#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define TRUE_PERMISSION TRUE
#define FALSE_PERMISSION FALSE
#else
#define TRUE_PERMISSION S_IRUGO
#define FALSE_PERMISSION S_IRUGO
#endif

// Static variables
/////////////////////////////////////////////////////////////////////////////

static int DMADescriptorsPerEngine = 8192;
static char *FirmwareFile = NULL;
static int logmask = 0x07;
static int MinBufferPoolItems = 16;
static int UseIntCtrl = 1;
static int UseMSI = 1;
static int UseMSIMulti = 1;
static int UseMSIX = 1;

MODULE_PARM_DESC(DMADescriptorsPerEngine, "Number of DMA Descriptors per Engine");
module_param(DMADescriptorsPerEngine, int, TRUE_PERMISSION);
MODULE_PARM_DESC(FirmwareFile, "Firmware file name");
module_param(FirmwareFile, charp, TRUE_PERMISSION);
MODULE_PARM_DESC(logmask, "Log mask: error=0x01, warning=0x02, info=0x04, debug=0x08, text=0x10");
module_param(logmask, int, TRUE_PERMISSION);
MODULE_PARM_DESC(MinBufferPoolItems, "Minimum buffer pool items");
module_param(MinBufferPoolItems, int, FALSE_PERMISSION);
MODULE_PARM_DESC(UseIntCtrl, "Use Interrupt Control [TRUE|FALSE]");
module_param(UseIntCtrl, int, TRUE_PERMISSION);
MODULE_PARM_DESC(UseMSI, "Use MSI [TRUE|FALSE]");
module_param(UseMSI, int, TRUE_PERMISSION);
MODULE_PARM_DESC(UseMSIMulti, "Use MSI Multi-Vector [TRUE|FALSE]");
module_param(UseMSIMulti, int, TRUE_PERMISSION);
MODULE_PARM_DESC(UseMSIX, "Use MSIX [TRUE|FALSE]");
module_param(UseMSIX, int, TRUE_PERMISSION);

// Functions
/////////////////////////////////////////////////////////////////////////////

int at_param_get(enum at_param_id aId)
{
    switch (aId)
    {
        case AT_PARAM_DMA_DESC_PER_ENGINE:
            return DMADescriptorsPerEngine;
        case AT_PARAM_LOG_MASK:
            return logmask;
        case AT_PARAM_MIN_BUFFER_POOL_ITEMS:
            return MinBufferPoolItems;
        case AT_PARAM_USE_INT_CTRL:
            return UseIntCtrl;
        case AT_PARAM_USE_MSI:
            return UseMSI;
        case AT_PARAM_USE_MSI_MULTI:
            return UseMSIMulti;
        case AT_PARAM_USE_MSIX:
            return UseMSIX;

        default:
            at_assert(false);
    }

    return 0;
}

const char *at_param_get_string(enum at_param_id aId)
{
    switch (aId)
    {
        case AT_PARAM_FIRMWARE_FILE:
            return FirmwareFile;

        default:
            at_assert(false);
    }

    return NULL;
}

void at_param_set(enum at_param_id aId, int aValue)
{
    switch (aId)
    {
        case AT_PARAM_USE_MSIX:
            UseMSIX = aValue;
            break;

        default:
            at_assert(false);
    }
}
