//=============================================================================
//
// Alazar Technologies Inc
//
// Copyright (c) 2019 Alazar Technologies Inc. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States And Canada.
//
// This product contains confidential information and trade secrets
// of Alazar Technologies Inc. Use, disclosure, or reproduction is
// prohibited without the prior express written permission of Alazar
// Technologies Inc
//
//=============================================================================

#include <linux/version.h>

#if LINUX_VERSION_CODE > KERNEL_VERSION(4, 10, 0)
#if defined(__x86_64__) || defined(_M_X64)
#include <asm/segment.h>
#endif
#endif

#if LINUX_VERSION_CODE > KERNEL_VERSION(4, 11, 0)
#include <linux/uaccess.h>
#else
#include <asm/uaccess.h>
#endif

#include <linux/fs.h>

#include "platform.h"

#include "ats_file.h"

#include "utils.h"

/******************************************************************************
 *
 * Function   :  ats_file_open
 *
 * Description:  Open file descriptor
 *
 ******************************************************************************/
ats_file_t ats_file_open(ats_char_t *path, bool wren)
{
#if 0
    struct file *filp = NULL;
    char cpath[1024];
    mm_segment_t oldfs;
    int err = 0;

    oldfs = get_fs();
    set_fs(get_ds());
    filp = filp_open(cpath, O_RDONLY, 0);
    set_fs(oldfs);

    if (IS_ERR(filp)) {
        err = PTR_ERR(filp);
        return NULL;
    }

    return (ats_file_t)filp;
#else
    return NULL;
#endif
}

/******************************************************************************
 *
 * Function   :  ats_file_close
 *
 * Description:  Close file descriptor
 *
 ******************************************************************************/
int ats_file_close(ats_file_t handle)
{
#if 0
    filp_close((struct file *)handle, NULL);
    return 0;
#else
    return -1;
#endif
}

/******************************************************************************
 *
 * Function   :  ats_file_read
 *
 * Description:  Read data from file
 *
 ******************************************************************************/
int32_t ats_file_read(ats_file_t handle, uint8_t *pbuf, int32_t size)
{
#if 0
    struct file *filp = (struct file *)handle;
    mm_segment_t oldfs;
    int ret;

    oldfs = get_fs();
    set_fs(get_ds());
    ret = vfs_read(filp, pbuf, size, &filp->f_pos);
    set_fs(oldfs);
    return ret;
#else
    return -1;
#endif
}

/******************************************************************************
 *
 * Function   :  ats_file_write
 *
 * Description:  Write data to file
 *
 ******************************************************************************/
int32_t ats_file_write(ats_file_t handle, uint8_t *pbuf, int32_t size)
{
#if 0
    struct file *filp = (struct file *)handle;
    mm_segment_t oldfs;
    int ret;

    oldfs = get_fs();
    set_fs(get_ds());
    ret = vfs_write(filp, pbuf, size, &filp->f_pos);
    set_fs(oldfs);
    return ret;
#else
    return -1;
#endif
}

/******************************************************************************
 *
 * Function   :  ats_file_seek
 *
 * Description:  Goto offset in file
 *
 ******************************************************************************/
int64_t ats_file_seek(ats_file_t handle, ats_file_seek_t seek, int64_t offset)
{
#if 0
    struct file *filp = (struct file *)handle;
    int64_t pos = filp->f_pos;
    int64_t end = filp->f_ra.size;

    switch (seek) {
    case ATS_FILE_SEEK_SET:
        pos = offset;
        break;

    case ATS_FILE_SEEK_CURR:
        pos += offset;
        break;

    case ATS_FILE_SEEK_END:
        pos = end - offset;
        break;

    default:
        return -1;
    }

    if ((pos < 0) || (pos >= end)) return -1;

    filp->f_pos = pos;
    return pos;
#else
    return -1;
#endif
}

/******************************************************************************
 *
 * Function   :  ats_dir_open
 *
 * Description:  Open directory
 *
 ******************************************************************************/
ats_dir_t ats_dir_open(ats_char_t *path) { return (ats_dir_t)NULL; }

/******************************************************************************
 *
 * Function   :  ats_dir_close
 *
 * Description:  Close directory descriptor
 *
 ******************************************************************************/
int ats_dir_close(ats_dir_t handle) { return 0; }

/******************************************************************************
 *
 * Function   :  ats_dir_find
 *
 * Description:  Search file pattern in directory
 *
 ******************************************************************************/
int ats_dir_find(ats_dir_t handle, ats_char_t *pattern, ats_char_t *found, int32_t size)
{
    return 0;
}
