/**
 * @file
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2019 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

#ifndef ATS_FILE_H
#define ATS_FILE_H

#include "ats_types.h"

typedef void *ats_file_t; ///< Handle file type
typedef void *ats_dir_t;  ///< Handle directory type

typedef enum
{
    ATS_FILE_SEEK_SET,
    ATS_FILE_SEEK_CURR,
    ATS_FILE_SEEK_END
} ats_file_seek_t;

/**
 * @brief Open file
 *
 * @param path      The path of file to open
 *
 * @param wren      Write enable
 *
 */
ats_file_t ats_file_open(ats_char_t *path, bool wren);

/**
 * @brief Close file
 *
 * @param file      The file to close returned by ats_file_open()
 *
 */
int ats_file_close(ats_file_t file);

/**
 * @brief Read file
 *
 * @param file      The file to read returned by ats_file_open()
 *
 * @param pbuf      Pointer to a buffer used to store read data
 *
 * @param size      Size of the data to read
 *
 */
int32_t ats_file_read(ats_file_t file, uint8_t *pbuf, int32_t size);

/**
 * @brief Write file
 *
 * @param file      The file to read returned by ats_file_open()
 *
 * @param pbuf      Pointer to a buffer of data to write
 *
 * @param size      Size of the data to write
 *
 */
int32_t ats_file_write(ats_file_t file, uint8_t *pbuf, int32_t size);

/**
 * @brief File seek
 *
 * @param file      The file to read returned by ats_file_open()
 *
 * @param seek      One of `ats_file_seek_t`
 *
 * @param offset    Offset to go to depending on seek type
 *
 */
int64_t ats_file_seek(ats_file_t handle, ats_file_seek_t seek, int64_t offset);

/**
 * @brief Open directory
 *
 * @param path      The path of directory to open
 *
 */
ats_dir_t ats_dir_open(ats_char_t *path);

/**
 * @brief Close directory
 *
 * @param dir       The directory to close returned by ats_dir_open()
 *
 */
int ats_dir_close(ats_dir_t dir);

/**
 * @brief Search files in a directory
 *
 * @param dir       The directory to close returned by ats_dir_open()
 *
 * @param pattern   Path search pattern
 *
 * @param found     Pointer to a buffer used to store the found entry name
 *
 * @param size      Size of the found buffer
 *
 */
int ats_dir_find(ats_dir_t dir, ats_char_t *pattern, ats_char_t *found, int32_t size);

#endif // ATS_FILE_H
