///
/// @file
///
/// This module is used to register the "proc" file system on Linux Kernel.
///
/// @author Alazar Technologies Inc
///
/// @copyright Copyright (c) 2021 Alazar Technologies Inc. All Rights Reserved.
///

// This file is included into the dkms package.

#ifndef ATS_PROCFS_H
#define ATS_PROCFS_H

#ifndef _WIN32

#include "ats_error.h"
#include "ats_types.h"

#include "at_linux_fs.h"
#include "at_linux_proc_fs.h"
#include "at_linux_seq_file.h"
#include "at_main.h"

#include "ats_dev.h"
#include "ats_pci.h"
#include "board_config.h"

typedef int (*ats_procfs_show_func_t)(void *context, struct seq_file *secfile);

typedef ssize_t (*ats_procfs_write_func_t)(void *context,
                                           const char __user *buffer,
                                           size_t length,
                                           loff_t *offset);

typedef struct
{
    const char *name;
    mode_t mode;
    ats_procfs_show_func_t show;
    ats_procfs_write_func_t write;
    void *context;
} ats_procfs_info_t;

typedef struct
{
    struct proc_dir_entry *rootdir;
} ats_procfs_t;

rc_t ats_procfs_create(ats_procfs_t *procfs, const char *dir);

rc_t ats_procfs_destroy(ats_procfs_t *procfs);

rc_t ats_procfs_add(ats_procfs_t *procfs,
                    const char *name,
                    mode_t mode,
                    ats_procfs_show_func_t show,
                    ats_procfs_write_func_t write,
                    void *context);

rc_t ats_procfs_attach(ats_procfs_t *procfs,
                       const char *dir,
                       ats_pci_t *pci,
                       board_config_database_t *config,
                       ats_dev_t *ats_dev);

rc_t ats_procfs_detach(ats_procfs_t *procfs);

#endif

#endif // ATS_PROCFS_H
