//=============================================================================
//
// Alazar Technologies Inc
//
// Copyright (c) 2003 Alazar Technologies Inc. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States And Canada.
//
// This product contains confidential information and trade secrets
// of Alazar Technologies Inc. Use, disclosure, or reproduction is
// prohibited without the prior express written permission of Alazar
// Technologies Inc
//
//=============================================================================

#include <linux/jiffies.h>
#include <linux/param.h>
#include <linux/sched.h>
#include <linux/version.h>

#include "ats_timer.h"

enum
{
    ATS_TIMER_DISABLED,
    ATS_TIMER_ENABLED,
    ATS_TIMER_AUTO
};

static void ats_timer_call(struct timer_list *pctx)
{
    ats_timer_t *ptim = container_of(pctx, ats_timer_t, timer);

    if (ptim->callback != NULL) ptim->callback(ptim->context);

    if (ptim->state == ATS_TIMER_AUTO) ats_timer_reset(ptim);
}

rc_t ats_timer_reset(ats_timer_t *ptim)
{
    if (ptim == NULL) return rc_null_param;

    mod_timer(&ptim->timer, jiffies + msecs_to_jiffies(ptim->delay));
    return rc_success;
}

rc_t ats_timer_start(
    ats_timer_t *ptim, bool restart, unsigned int delay, void (*callback)(void *), void *context)
{
    if ((ptim == NULL) || ((ptim->callback == NULL) && (callback == NULL))) return rc_null_param;

    if (callback != NULL)
    {
        ptim->callback = callback;
        ptim->context = context;
    }

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 15, 0)
    init_timer(&ptim->timer);
    ptim->timer.function = (void (*)(unsigned long))ats_timer_call;
    ptim->timer.data = (unsigned long)&ptim->timer;
#else
    timer_setup(&ptim->timer, (void (*)(struct timer_list *))ats_timer_call, 0);
#endif
    ptim->state = restart ? ATS_TIMER_AUTO : ATS_TIMER_ENABLED;
    ptim->delay = delay;
    return ats_timer_reset(ptim);
}

rc_t ats_timer_stop(ats_timer_t *ptim)
{
    if (ptim == NULL) return rc_null_param;

    ptim->state = ATS_TIMER_DISABLED;
    del_timer(&ptim->timer);
    return rc_success;
}
