/**
 * @file
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2021 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

#ifndef ATS_TIMER_H
#define ATS_TIMER_H

#include "ats_error.h"

#ifdef _WIN32
#include <ntddk.h>
#include <wdf.h>
#else
#ifdef MODULE
#include <linux/timer.h>
#endif
#endif

typedef struct ats_timer_s
{
#ifdef _WIN32
    WDF_TIMER_CONFIG config;     ///<
    WDF_OBJECT_ATTRIBUTES attrs; ///<
    WDFTIMER timer;              ///<
#else
    union
    {
#ifdef MODULE
        struct timer_list timer; ///<
#endif
        char reserved0[128];
    };
#endif
    unsigned int delay;       ///<
    int state;                ///<
    void (*callback)(void *); ///<
    void *context;            ///<
} ats_timer_t;

/**
 *  @brief Initialize and start timer.
 *
 *  @param ptim     The timer context structure pointer
 *
 *  @param restart  The timer restart automaticaly if true
 *
 *  @param delay    The timer delay in milli-seconds
 *
 *  @param callback The function called when the timer expire
 *
 *  @param context  The context pointer used by the callback() function
 *
 */
rc_t ats_timer_start(
    ats_timer_t *ptim, bool restart, unsigned int delay, void (*callback)(void *), void *context);

/**
 *  @brief Stop and disable timer.
 *
 *  @param ptim     The timer context structure pointer
 *
 */
rc_t ats_timer_stop(ats_timer_t *ptim);

/**
 *  @brief Restart timer.
 *
 *  @param ptim     The timer context structure pointer
 *
 */
rc_t ats_timer_reset(ats_timer_t *ptim);

#endif // ATS_TIMER_H
