//=============================================================================
//
// Alazar Technologies Inc
//
// Copyright (c) 2003 Alazar Technologies Inc. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States And Canada.
//
// This product contains confidential information and trade secrets
// of Alazar Technologies Inc. Use, disclosure, or reproduction is
// prohibited without the prior express written permission of Alazar
// Technologies Inc
//
//=============================================================================

// This file is included into the dkms package.

// Includes
/////////////////////////////////////////////////////////////////////////////

// ===== Linux kernel =======================================================
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/stddef.h>
#include <linux/uaccess.h>
#include <linux/version.h>
#include <linux/vmalloc.h>

#include <asm/io.h>

// ===== src ================================================================
#include "platform.h"

// ===== src/ats... =========================================================
#include <DeviceId.h>

void ats_write_register_u32(uint32_t *pmem, uint32_t val) { writel(val, pmem); }

uint32_t ats_read_register_u32(uint32_t *pmem) { return readl(pmem); }

int64_t ats_get_utime(void)
{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 10, 0))
#include <linux/time.h>
    struct timeval now;
    do_gettimeofday(&now);
    return (((int64_t)now.tv_sec * 1000000) + now.tv_usec);
#else
#include <linux/ktime.h>
    return (int64_t)(ktime_get_boot_fast_ns() / 1000);
#endif
}

void ats_udelay(unsigned long delay) { udelay(delay); }

void ats_mdelay(unsigned long delay) { mdelay(delay); }

void *ats_alloc(size_t size_bytes) { return kmalloc(size_bytes, GFP_KERNEL); }

void ats_free(void *buffer) { kfree(buffer); }

void *ats_alloc_large_buffer(size_t size_bytes) { return vmalloc(size_bytes); }

void ats_free_large_buffer(void *buffer) { vfree(buffer); }

static uint32_t ats_log_mask_current = ATS_MASK(ATS_LOG_ERROR) | ATS_MASK(ATS_LOG_WARNING);

void ats_log_init(uint32_t mask) { ats_log_mask_current = mask; }

uint32_t ats_log_mask(void) { return ats_log_mask_current; }

void ats_log(int level, const char *format, ...)
{
    char header[256];
    va_list list;

    if ((level < 0) || (((ats_log_mask_current >> level) & 1) == 0)) return;

    switch (level)
    {
        case ATS_LOG_ERROR:
            strncpy(header, KERN_ERR MTAG ": ", sizeof(header) - 1);
            break;

        case ATS_LOG_WARNING:
            strncpy(header, KERN_WARNING MTAG ": ", sizeof(header) - 1);
            break;

        case ATS_LOG_INFO:
            strncpy(header, KERN_INFO MTAG ": ", sizeof(header) - 1);
            break;

        case ATS_LOG_DEBUG:
            strncpy(header, KERN_DEBUG MTAG ": ", sizeof(header) - 1);
            break;

        default:
            return;
    }

    strncat(header, format, sizeof(header) - strlen(header) - 1);
    va_start(list, format);
    vprintk(header, list);
    va_end(list);
}

int ats_copy_to_user(void *dst, void *src, size_t size_bytes)
{
    return copy_to_user(dst, src, size_bytes);
}

int ats_copy_from_user(void *dst, void *src, size_t size_bytes)
{
    return copy_from_user(dst, src, size_bytes);
}

bool ats_access_ok(void *addr, size_t size)
{
#if defined(__range_not_ok)
    return likely(!__range_not_ok((long unsigned int)addr, size, user_addr_max()));
#elif defined(__range_ok)
    return likely(__range_ok((long unsigned int)addr, size));
#else
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
    return likely(access_ok(VERIFY_WRITE, (long unsigned int)addr, size));
#else
    return likely(access_ok((long unsigned int)addr, size));
#endif
#endif
}
